# The AFTER command

# $Id: after.t,v 3.0.1.1 1994/09/22 14:40:57 ram Exp $
#
#  Copyright (c) 1990-1993, Raphael Manfredi
#  
#  You may redistribute only under the terms of the Artistic License,
#  as specified in the README file that comes with the distribution.
#  You may reuse parts of this distribution only within the terms of
#  that same Artistic License; a copy of which may be found at the root
#  of the source tree for mailagent 3.0.
#
# $Log: after.t,v $
# Revision 3.0.1.1  1994/09/22 14:40:57  ram
# patch12: created
#

print "-1\n";	# Automatic testing difficult
